/**
 *    DefaultFrameReader.java
 *    Copyright (C) 2010 New Zealand Digital Library, http://expeditee.org
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.expeditee.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.lang.reflect.Method;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.LinkedList;

import org.expeditee.core.Colour;
import org.expeditee.core.Font;
import org.expeditee.core.Point;
import org.expeditee.gui.Frame;
import org.expeditee.items.Constraint;
import org.expeditee.items.Dot;
import org.expeditee.items.DotType;
import org.expeditee.items.Item;
import org.expeditee.items.Justification;
import org.expeditee.items.Line;
import org.expeditee.items.PermissionPair;
import org.expeditee.items.Text;

public abstract class DefaultFrameReader implements FrameReader {
	
	protected static LinkedHashMap<Character, Method> _ItemTags = null;
	
	protected static List<Character> _DelayedItemTags = null;

	protected static LinkedHashMap<Character, Method> _FrameTags = null;
	
	protected static Class[] pString = { String.class };
	protected static Class[] pInt = { int.class };
	protected static Class[] pIntO = { Integer.class };
	protected static Class[] pFloat = { float.class };
	protected static Class[] pFloatO = { Float.class };
	protected static Class[] pDouble = { double.class };
	protected static Class[] pColor = { Colour.class };
	protected static Class[] pBool = { boolean.class };
	protected static Class[] pFont = { Font.class };
	protected static Class[] pPoint = { Point.class };
	protected static Class[] pArrow = { float.class, double.class, double.class };
	protected static Class[] pList = { List.class };
	protected static Class[] pIntArray = { int[].class };
	protected static Class[] pItem = { Item.class };
	protected static Class[] pJustification = { Justification.class };
	protected static Class[] pPermission = { PermissionPair.class };
	protected static Class[] pDotType = { DotType.class };
	
	public DefaultFrameReader(){
		if (_ItemTags != null && _FrameTags != null)
			return;

		_ItemTags = new LinkedHashMap<Character, Method>();
		_FrameTags = new LinkedHashMap<Character, Method>();
		_DelayedItemTags = new LinkedList<Character>();

		try {
			_FrameTags.put('A', Frame.class.getMethod("setName", pString));
			_FrameTags.put('V', Frame.class.getMethod("setVersion", pInt));
			_FrameTags.put('p', Frame.class.getMethod("setPermission", pPermission));
			_FrameTags.put('U', Frame.class.getMethod("setOwner", pString));
			_FrameTags.put('D', Frame.class.getMethod("setDateCreated", pString));
			_FrameTags.put('M', Frame.class.getMethod("setLastModifyUser", pString));
			_FrameTags.put('d', Frame.class.getMethod("setLastModifyDate", pString));
			_FrameTags.put('F', Frame.class.getMethod("setFrozenDate", pString));
			_FrameTags.put('O', Frame.class.getMethod("setForegroundColor",	pColor));
			_FrameTags.put('B', Frame.class.getMethod("setBackgroundColor",	pColor));
			_FrameTags.put('K', Frame.class.getMethod("setEncryptionLabel", pString));

			// Note: As of 26/11/18 there are no unused letter item tags.  Use other characters.
			_ItemTags.put('S', Item.class.getMethod("setID", pInt));
			_ItemTags.put('s', Item.class.getMethod("setDateCreated", pString));
			_ItemTags.put('d', Item.class.getMethod("setColor", pColor));
			_ItemTags.put('G', Item.class.getMethod("setBackgroundColor",
					pColor));
			_ItemTags.put('K', Item.class.getMethod("setBorderColor",
					pColor));

			_ItemTags.put('R', Item.class.getMethod("setAnchorLeft", pIntO));
			_ItemTags.put('H', Item.class.getMethod("setAnchorRight", pIntO));
			_ItemTags.put('N', Item.class.getMethod("setAnchorTop", pIntO));
			_ItemTags.put('I', Item.class.getMethod("setAnchorBottom", pIntO));

			_ItemTags.put('P', Item.class.getMethod("setPosition", pPoint));
			_ItemTags.put('F', Item.class.getMethod("setLink", pString));
			_ItemTags.put('J', Item.class.getMethod("setFormula", pString));
			
			_ItemTags.put('X', Item.class.getMethod("setActions", pList));
			_ItemTags.put('x', Item.class.getMethod("setActionMark", pBool));
			_ItemTags.put('U', Item.class.getMethod("setActionCursorEnter",
					pList));
			_ItemTags.put('V', Item.class.getMethod("setActionCursorLeave",
					pList));
			_ItemTags.put('W', Item.class.getMethod("setActionEnterFrame",
					pList));
			_ItemTags.put('Y', Item.class.getMethod("setActionLeaveFrame",
					pList));
			_ItemTags.put('D', Item.class.getMethod("addToData", pString));
			_ItemTags.put('u', Item.class.getMethod("setHighlight", pBool));
			_ItemTags.put('e', Item.class.getMethod("setFillColor", pColor));
			_ItemTags.put('E', Item.class.getMethod("setGradientColor", pColor));
			_ItemTags.put('Q', Item.class.getMethod("setGradientAngle", pDouble));
			
			_ItemTags.put('i', Item.class.getMethod("setFillPattern", pString));
			_ItemTags.put('o', Item.class.getMethod("setOwner", pString));
			_ItemTags.put('n', Item.class.getMethod("setLinkMark", pBool));
			_ItemTags
					.put('q', Item.class.getMethod("setLinkFrameset", pString));
			_ItemTags
					.put('y', Item.class.getMethod("setLinkTemplate", pString));
			_ItemTags.put('g', Item.class.getMethod("setLinePattern", pIntArray));

			_ItemTags.put('j', Item.class.getMethod("setArrow", pArrow));

			_ItemTags.put('v', Dot.class.getMethod("setDotType", pDotType));
			_ItemTags.put('z', Dot.class.getMethod("setFilled", pBool));
			
			_ItemTags.put('f', Text.class.getMethod("setFont", pFont));
			_ItemTags.put('t', Text.class.getMethod("setSpacing", pFloat));
			_ItemTags.put('T', Text.class.getMethod("appendLine", pString));
			_ItemTags.put('a', Text.class.getMethod("setWordSpacing", pInt));
			_ItemTags.put('b', Text.class.getMethod("setLetterSpacing", pFloat));
			_ItemTags.put('m', Text.class.getMethod("setInitialSpacing", pFloat));
			_ItemTags.put('w', Text.class.getMethod("setWidth", pIntO));
			_ItemTags.put('M', Text.class.getMethod("setMinWidth", pIntO));
			_ItemTags.put('k', Text.class.getMethod("setJustification", pJustification));
			_ItemTags.put('r', Text.class.getMethod("setAutoWrap", pBool));

			_ItemTags.put('h', Item.class.getMethod("setThickness", pFloat));
			_ItemTags.put('l', Item.class.getMethod("setLineIDs", pString));
			_ItemTags.put('c', Item.class.getMethod("setConstraintIDs", pString));
			
			_ItemTags.put('A', Item.class.getMethod("setTooltip", pString));
			_ItemTags.put('B', Item.class.getMethod("setLinkHistory", pBool));
			
			_ItemTags.put('p', Item.class.getMethod("setPermission", pPermission));
			
			_ItemTags.put('O', Text.class.getMethod("setMask", pIntO));
			
			// Lines and constraints are created differently
			_ItemTags.put('L', Line.class.getMethod("setStartItem", pItem));
			_ItemTags.put('C', Constraint.class.getMethod("getID", (Class[]) null));
			
			_ItemTags.put('[', Item.class.getMethod("setMagnetizedItemLeft", pInt));
			_DelayedItemTags.add('[');
			_ItemTags.put(']', Item.class.getMethod("setMagnetizedItemRight", pInt));
			_DelayedItemTags.add(']');
			_ItemTags.put('^', Item.class.getMethod("setMagnetizedItemTop", pInt));
			_DelayedItemTags.add('^');
			_ItemTags.put('_', Item.class.getMethod("setMagnetizedItemBottom", pInt));
			_DelayedItemTags.add('_');
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	public Frame readFrame(String fullPath) throws IOException {
		File f = new File(fullPath);
		Reader in = new InputStreamReader(new FileInputStream(fullPath), "UTF-8");
		Frame frame = readFrame(new BufferedReader(in));
		frame.setLastModifyDate(frame.getLastModifyDate(), f.lastModified());
		return frame;
	}
}
